@@ -19,6 +19,7 @@ BUILD_CONTAINER_FQDN_IMAGE = REGISTRY + '/' + BUILD_CONTAINER_IMAGE + ':' + BUIL
 
 ENV = ["KEY1=VAL1", "KEY2=VAL2"]
 ENV_FILE_PATH = '/home/envfile.env'
+ENV_FILES = ['/home/envfile1.env', '/home/envfile2.env']
 
 SKIPPER_CONF_CONTAINER_CONTEXT = '/some/context'
 SKIPPER_CONF_BUILD_CONTAINER_IMAGE = 'skipper-conf-build-container-image'
@@ -61,7 +62,16 @@ SKIPPER_CONF_WITH_ENV_FILE = {
     'make': {
         'makefile': SKIPPER_CONF_MAKEFILE,
     },
-    'env-file': ENV_FILE_PATH
+    'env_file': [ENV_FILE_PATH]
+}
+SKIPPER_CONF_WITH_MULTIPLE_ENV_FILES = {
+    'registry': REGISTRY,
+    'build-container-image': SKIPPER_CONF_BUILD_CONTAINER_IMAGE,
+    'build-container-tag': SKIPPER_CONF_BUILD_CONTAINER_TAG,
+    'make': {
+        'makefile': SKIPPER_CONF_MAKEFILE,
+    },
+    'env_file': ENV_FILES
 }
 SKIPPER_CONF_WITH_ENV_LIST = {
     'registry': REGISTRY,
@@ -153,6 +163,33 @@ SKIPPER_CONF_WITH_CONTEXT_NO_TAG = {
     'container-context': SKIPPER_CONF_CONTAINER_CONTEXT
 }
 
+SKIPPER_CONF_WITH_SHELL_INTERPOLATION = {
+    'registry': REGISTRY,
+    'build-container-image': SKIPPER_CONF_BUILD_CONTAINER_IMAGE,
+    'build-container-tag': SKIPPER_CONF_BUILD_CONTAINER_TAG,
+    'make': {
+        'makefile': SKIPPER_CONF_MAKEFILE,
+    },
+    'volumes': [
+        '$(which cat):/cat',
+    ],
+    'env': [
+        'KEY=$(expr ${MY_NUMBER:-5} + 5)'
+    ]
+}
+
+SKIPPER_CONF_WITH_INVALID_SHELL_INTERPOLATION = {
+    'registry': REGISTRY,
+    'build-container-image': SKIPPER_CONF_BUILD_CONTAINER_IMAGE,
+    'build-container-tag': SKIPPER_CONF_BUILD_CONTAINER_TAG,
+    'make': {
+        'makefile': SKIPPER_CONF_MAKEFILE,
+    },
+    'volumes': [
+        '$(bla bla):/cat',
+    ]
+}
+
 
 class TestCLI(unittest.TestCase):
     def setUp(self):
@@ -304,7 +341,7 @@ class TestCLI(unittest.TestCase):
                        SKIPPER_CONF_CONTAINER_CONTEXT]),
             mock.call(['make'] + make_params, fqdn_image='build-container-image', environment=[],
                       interactive=False, name=None, net=None, publish=(), volumes=None, workdir=None,
-                      use_cache=False, workspace=None, env_file=None),
+                      use_cache=False, workspace=None, env_file=()),
         ]
         skipper_runner_run_mock.assert_has_calls(expected_commands)
 
@@ -1042,7 +1079,7 @@ class TestCLI(unittest.TestCase):
                                                         interactive=False, name=None, net=None, publish=(),
                                                         volumes=None,
                                                         workdir=None, workspace=None, use_cache=False,
-                                                        env_file=None)
+                                                        env_file=())
 
     @mock.patch('subprocess.check_output', mock.MagicMock(autospec=True, return_value=''))
     @mock.patch('requests.get', autospec=True)
@@ -1069,7 +1106,7 @@ class TestCLI(unittest.TestCase):
                                                         interactive=False, name=None, net=None, publish=(),
                                                         volumes=None,
                                                         workdir=None, workspace=None, use_cache=False,
-                                                        env_file=None)
+                                                        env_file=())
 
     @mock.patch('subprocess.check_output', mock.MagicMock(autospec=True, return_value=''))
     @mock.patch('skipper.runner.run', mock.MagicMock(autospec=True))
@@ -1110,7 +1147,7 @@ class TestCLI(unittest.TestCase):
                                                         interactive=False, name=None, net=None, publish=(),
                                                         volumes=None,
                                                         workdir=None, workspace=None, use_cache=False,
-                                                        env_file=None)
+                                                        env_file=())
 
     @mock.patch('__builtin__.open', mock.MagicMock(create=True))
     @mock.patch('os.path.exists', mock.MagicMock(autospec=True, return_value=True))
@@ -1132,7 +1169,7 @@ class TestCLI(unittest.TestCase):
                                                         interactive=False, name=None, net=None, publish=(),
                                                         volumes=None,
                                                         workdir=None, workspace=None, use_cache=False,
-                                                        env_file=None)
+                                                        env_file=())
 
     @mock.patch('__builtin__.open', mock.MagicMock(create=True))
     @mock.patch('os.path.exists',
@@ -1163,7 +1200,38 @@ class TestCLI(unittest.TestCase):
                                                         workdir=None,
                                                         workspace=None,
                                                         use_cache=False,
-                                                        env_file=ENV_FILE_PATH)
+                                                        env_file=(ENV_FILE_PATH,))
+
+    @mock.patch('__builtin__.open', mock.MagicMock(create=True))
+    @mock.patch('os.path.exists',
+                mock.MagicMock(autospec=True, return_value=True))
+    @mock.patch('yaml.safe_load', mock.MagicMock(
+        autospec=True, return_value=SKIPPER_CONF_WITH_MULTIPLE_ENV_FILES))
+    @mock.patch('subprocess.check_output',
+                mock.MagicMock(autospec=True, return_value='1234567\n'))
+    @mock.patch('skipper.runner.run', autospec=True)
+    def test_run_with_defaults_and_env_from_multiple_env_file(
+            self,
+            skipper_runner_run_mock
+    ):
+        command = ['ls', '-l']
+        run_params = command
+        self._invoke_cli(
+            defaults=config.load_defaults(),
+            subcmd='run',
+            subcmd_params=run_params
+        )
+        expected_fqdn_image = 'skipper-conf-build-container-image:skipper-conf-build-container-tag'
+        skipper_runner_run_mock.assert_called_once_with(command,
+                                                        fqdn_image=expected_fqdn_image,
+                                                        environment=[],
+                                                        interactive=False,
+                                                        name=None, net=None, publish=(),
+                                                        volumes=None,
+                                                        workdir=None,
+                                                        workspace=None,
+                                                        use_cache=False,
+                                                        env_file=tuple(ENV_FILES))
 
     @mock.patch('__builtin__.open', mock.MagicMock(create=True))
     @mock.patch('os.path.exists', mock.MagicMock(autospec=True, return_value=True))
@@ -1185,7 +1253,7 @@ class TestCLI(unittest.TestCase):
                                                         interactive=False, name=None, net=None, publish=(),
                                                         volumes=None,
                                                         workdir=None, workspace=None, use_cache=False,
-                                                        env_file=None)
+                                                        env_file=())
 
     @mock.patch('__builtin__.open', mock.MagicMock(create=True))
     @mock.patch('os.path.exists', mock.MagicMock(autospec=True, return_value=True))
@@ -1208,7 +1276,7 @@ class TestCLI(unittest.TestCase):
                                                         interactive=False, name=None, net=None, publish=(),
                                                         volumes=None,
                                                         workdir=None, workspace=None, use_cache=False,
-                                                        env_file=None)
+                                                        env_file=())
 
     @mock.patch('__builtin__.open', mock.MagicMock(create=True))
     @mock.patch('os.path.exists', mock.MagicMock(autospec=True, return_value=True))
@@ -1231,7 +1299,7 @@ class TestCLI(unittest.TestCase):
                                                         interactive=False, name=None, net=None, publish=(),
                                                         volumes=None,
                                                         workdir=None, workspace=None, use_cache=False,
-                                                        env_file=None)
+                                                        env_file=())
 
     @mock.patch('__builtin__.open', mock.MagicMock(create=True))
     @mock.patch('os.path.exists', mock.MagicMock(autospec=True, return_value=True))
@@ -1265,7 +1333,7 @@ class TestCLI(unittest.TestCase):
                                                         interactive=False, name=None, net=None, publish=(),
                                                         volumes=None,
                                                         workdir=None, workspace=None, use_cache=False,
-                                                        env_file=None)
+                                                        env_file=())
 
     @mock.patch('subprocess.check_output', mock.MagicMock(autospec=True, return_value='1234567\n'))
     @mock.patch('skipper.runner.run', autospec=True)
@@ -1282,7 +1350,7 @@ class TestCLI(unittest.TestCase):
         skipper_runner_run_mock.assert_called_once_with(command, fqdn_image=expected_fqdn_image, environment=[],
                                                         interactive=True, name=None, net=None, publish=(), volumes=None,
                                                         workdir=None, workspace=None, use_cache=False,
-                                                        env_file=None)
+                                                        env_file=())
         del os.environ['SKIPPER_INTERACTIVE']
 
     @mock.patch('subprocess.check_output', mock.MagicMock(autospec=True, return_value='1234567\n'))
@@ -1301,7 +1369,7 @@ class TestCLI(unittest.TestCase):
                                                         interactive=False, name=None, net=None, publish=(),
                                                         volumes=None,
                                                         workdir=None, workspace=None, use_cache=False,
-                                                        env_file=None)
+                                                        env_file=())
         del os.environ['SKIPPER_INTERACTIVE']
 
     @mock.patch('subprocess.check_output', mock.MagicMock(autospec=True, return_value='1234567\n'))
@@ -1318,7 +1386,7 @@ class TestCLI(unittest.TestCase):
         skipper_runner_run_mock.assert_called_once_with(command, fqdn_image=expected_fqdn_image, environment=[],
                                                         interactive=True, name=None, net=None, publish=(), volumes=None,
                                                         workdir=None, workspace=None, use_cache=False,
-                                                        env_file=None)
+                                                        env_file=())
 
     @mock.patch('subprocess.check_output', mock.MagicMock(autospec=True, return_value=''))
     @mock.patch('skipper.runner.run', autospec=True, return_value=0)
@@ -1336,7 +1404,7 @@ class TestCLI(unittest.TestCase):
                        'Dockerfile.build-container-image', '.']),
             mock.call(command, fqdn_image='build-container-image', environment=[],
                       interactive=False, name=None, net=None, publish=(), volumes=None, workdir=None, workspace=None,
-                      use_cache=False, env_file=None),
+                      use_cache=False, env_file=()),
         ]
         skipper_runner_run_mock.assert_has_calls(expected_commands)
 
@@ -1354,7 +1422,7 @@ class TestCLI(unittest.TestCase):
         expected_commands = [
             mock.call(command, fqdn_image='build-container-image', environment=[],
                       interactive=False, name=None, net=None, publish=(), volumes=None, workdir=None, workspace=None,
-                      use_cache=True, env_file=None),
+                      use_cache=True, env_file=()),
         ]
         skipper_runner_run_mock.assert_has_calls(expected_commands)
 
@@ -1374,7 +1442,7 @@ class TestCLI(unittest.TestCase):
         skipper_runner_run_mock.assert_called_once_with(command, fqdn_image=expected_fqdn_image, environment=[],
                                                         interactive=False, name=None, net='non-default-net',
                                                         publish=(), volumes=None, workdir=None, workspace=None,
-                                                        use_cache=False, env_file=None)
+                                                        use_cache=False, env_file=())
 
     @mock.patch('subprocess.check_output', mock.MagicMock(autospec=True, return_value='1234567'))
     @mock.patch('skipper.runner.run', autospec=True)
@@ -1400,7 +1468,7 @@ class TestCLI(unittest.TestCase):
                                                         interactive=False, name=None, net=u'non-default-net',
                                                         publish=(u'123:123',), volumes=None,
                                                         workdir=None, workspace=None, use_cache=False,
-                                                        env_file=None)
+                                                        env_file=())
 
     @mock.patch('subprocess.check_output', mock.MagicMock(autospec=True, return_value='1234567'))
     @mock.patch('skipper.runner.run', autospec=True)
@@ -1426,7 +1494,7 @@ class TestCLI(unittest.TestCase):
                                                         interactive=False, name=None, net=u'non-default-net',
                                                         publish=('123:123', '12:12'), volumes=None,
                                                         workdir=None, workspace=None, use_cache=False,
-                                                        env_file=None)
+                                                        env_file=())
 
     @mock.patch('subprocess.check_output', mock.MagicMock(autospec=True, return_value='1234567'))
     @mock.patch('skipper.runner.run', autospec=True)
@@ -1452,7 +1520,7 @@ class TestCLI(unittest.TestCase):
                                                         interactive=False, name=None, net=u'non-default-net',
                                                         publish=('123:123', '12-13:12-13'), volumes=None,
                                                         workdir=None, workspace=None, use_cache=False,
-                                                        env_file=None)
+                                                        env_file=())
 
     def test_run_with_publish_textual_port(self):
         global_params = self.global_params
@@ -1529,7 +1597,38 @@ class TestCLI(unittest.TestCase):
         skipper_runner_run_mock.assert_called_once_with(command, fqdn_image=expected_fqdn_image, environment=[],
                                                         interactive=False, name=None, net=None, publish=(),
                                                         volumes=['volume1', 'volume2'], workspace=None,
-                                                        workdir=None, use_cache=False, env_file=None)
+                                                        workdir=None, use_cache=False, env_file=())
+
+    @mock.patch('__builtin__.open', mock.MagicMock(create=True))
+    @mock.patch('os.path.exists', mock.MagicMock(autospec=True, return_value=True))
+    @mock.patch('yaml.safe_load', mock.MagicMock(autospec=True, return_value=SKIPPER_CONF_WITH_SHELL_INTERPOLATION))
+    @mock.patch('subprocess.check_output', mock.MagicMock(autospec=True, return_value='1234567\n'))
+    @mock.patch('skipper.runner.run', autospec=True)
+    def test_run_with_defaults_from_config_file_including_interpolated_volumes(self, skipper_runner_run_mock):
+        command = ['ls', '-l']
+        run_params = command
+        self._invoke_cli(
+            defaults=config.load_defaults(),
+            subcmd='run',
+            subcmd_params=run_params
+        )
+        expected_fqdn_image = 'skipper-conf-build-container-image:skipper-conf-build-container-tag'
+        skipper_runner_run_mock.assert_called_once_with(command, fqdn_image=expected_fqdn_image, environment=['KEY=10'],
+                                                        interactive=False, name=None, net=None, publish=(),
+                                                        volumes=['/bin/cat:/cat'], workspace=None,
+                                                        workdir=None, use_cache=False, env_file=())
+
+    @mock.patch('yaml.safe_load', mock.MagicMock(autospec=True, return_value=SKIPPER_CONF_WITH_INVALID_SHELL_INTERPOLATION))
+    def test_run_with_defaults_from_config_file_including_invalid_interploated_volumes_interpolated(self):
+        command = ['ls', '-l']
+        run_params = command
+
+        with self.assertRaises(ValueError):
+            self._invoke_cli(
+                defaults=config.load_defaults(),
+                subcmd='run',
+                subcmd_params=run_params
+            )
 
     @mock.patch('__builtin__.open', mock.MagicMock(create=True))
     @mock.patch('os.path.exists', mock.MagicMock(autospec=True, return_value=True))
@@ -1549,7 +1648,7 @@ class TestCLI(unittest.TestCase):
                                                         interactive=False, name=None, net=None, publish=(),
                                                         volumes=None,
                                                         workdir='test-workdir', workspace=None, use_cache=False,
-                                                        env_file=None)
+                                                        env_file=())
 
     @mock.patch('__builtin__.open', mock.MagicMock(create=True))
     @mock.patch('os.path.exists', mock.MagicMock(autospec=True, return_value=True))
@@ -1569,7 +1668,7 @@ class TestCLI(unittest.TestCase):
                                                         interactive=False, name=None, net=None, publish=(),
                                                         volumes=None,
                                                         workdir=None, workspace="/test/workspace", use_cache=False,
-                                                        env_file=None)
+                                                        env_file=())
 
     @mock.patch('__builtin__.open', mock.MagicMock(create=True))
     @mock.patch('os.path.exists', mock.MagicMock(autospec=True, return_value=True))
@@ -1590,7 +1689,7 @@ class TestCLI(unittest.TestCase):
                                                         interactive=False, name=None, net=None, publish=(),
                                                         volumes=None,
                                                         workdir=None, use_cache=False, workspace=None,
-                                                        env_file=None)
+                                                        env_file=())
 
     @mock.patch('__builtin__.open', mock.MagicMock(create=True))
     @mock.patch('os.path.exists', mock.MagicMock(autospec=True, return_value=True))
@@ -1611,7 +1710,7 @@ class TestCLI(unittest.TestCase):
                                                         interactive=False, name=None, net=None, publish=(),
                                                         volumes=None,
                                                         workdir=None, workspace=None, use_cache=False,
-                                                        env_file=None)
+                                                        env_file=())
 
     @mock.patch('subprocess.check_output', mock.MagicMock(autospec=True, return_value='1234567\n'))
     @mock.patch('skipper.runner.run', autospec=True)
@@ -1631,7 +1730,7 @@ class TestCLI(unittest.TestCase):
                                                         interactive=False, name=None, net=None, publish=(),
                                                         volumes=None,
                                                         workdir=None, workspace=None, use_cache=False,
-                                                        env_file=None)
+                                                        env_file=())
 
     @mock.patch('subprocess.check_output', mock.MagicMock(autospec=True, return_value='1234567\n'))
     @mock.patch('skipper.runner.run', autospec=True)
@@ -1647,7 +1746,7 @@ class TestCLI(unittest.TestCase):
                                                         interactive=False, name=None, net=None, publish=(),
                                                         volumes=None,
                                                         workdir=None, workspace=None, use_cache=False,
-                                                        env_file=None)
+                                                        env_file=())
 
     @mock.patch('subprocess.check_output', mock.MagicMock(autospec=True, return_value='1234567\n'))
     @mock.patch('skipper.runner.run', autospec=True)
@@ -1666,7 +1765,7 @@ class TestCLI(unittest.TestCase):
                                                         interactive=False, name=None, net=None, publish=(),
                                                         volumes=None,
                                                         workdir=None, workspace=None, use_cache=False,
-                                                        env_file=None)
+                                                        env_file=())
 
     @mock.patch('__builtin__.open', mock.MagicMock(create=True))
     @mock.patch('os.path.exists', mock.MagicMock(autospec=True, return_value=True))
@@ -1689,7 +1788,7 @@ class TestCLI(unittest.TestCase):
                                                         interactive=False, name=None, net=None, publish=(),
                                                         volumes=None,
                                                         workdir=None, workspace=None, use_cache=False,
-                                                        env_file=None)
+                                                        env_file=())
 
     @mock.patch('subprocess.check_output', mock.MagicMock(autospec=True, return_value=''))
     @mock.patch('skipper.runner.run', autospec=True, return_value=0)
@@ -1708,7 +1807,7 @@ class TestCLI(unittest.TestCase):
                        'Dockerfile.build-container-image', '.']),
             mock.call(['make'] + make_params, fqdn_image='build-container-image', environment=[],
                       interactive=False, name=None, net=None, publish=(), volumes=None, workdir=None, workspace=None,
-                      use_cache=False, env_file=None),
+                      use_cache=False, env_file=()),
         ]
         skipper_runner_run_mock.assert_has_calls(expected_commands)
 
@@ -1723,7 +1822,7 @@ class TestCLI(unittest.TestCase):
         skipper_runner_run_mock.assert_called_once_with(['bash'], fqdn_image=expected_fqdn_image, environment=[],
                                                         interactive=True, name=None, net=None, publish=(), volumes=None,
                                                         workdir=None, workspace=None, use_cache=False,
-                                                        env_file=None)
+                                                        env_file=())
 
     @mock.patch('click.echo', autospec=True)
     @mock.patch('skipper.cli.get_distribution', autospec=True)
